STATUSES = {
        200: "200 OK",
        201: "201 Created",
        202: "202 Accepted",
        300: "300 Multiple Choices",
        301: "301 Moved Permanently",
        302: "302 Found",
        304: "304 Not Modified",
        400: "400 Bad Request",
        401: "401 Unauthorized",
        403: "403 Forbidden",
        404: "404 Not Found",
        405: "405 Method Not Allowed",
        408: "408 Request Timeout",
        500: "500 Internal Server Error",
        501: "501 Not Implemented",
        577: "577 Unknown Status",
}


